{% extends "powerwiki/base.html" %}

{% block wiki_content %}

<form action="." method="POST" class="powerwiki__asset_form" enctype="multipart/form-data">
{% csrf_token %}

{{ form.wiki }}
{{ form.name }}
{{ form.non_field_errors }}

<p>
    {{ form.image }}
    {{ form.image.errors }}
</p>

<p class="powerwiki__asset_form-save">
    <input type="submit" value="Save" />
</p>
</form>

{% endblock %}
