{% extends "powerwiki/base.html" %}

{% block wiki_menu %}
    <ul class="powerwiki__menu_options">
        <li>
            <form action="{% url "powerwiki:search" wiki_slug=wiki_slug %}" method="GET">
            <input type="text" name="q" placeholder="Search" value="{{ search_query }}">
            <input type="submit" value="Go">
            </form>
        </li>
    </ul>
{% endblock %}

{% block wiki_content %}
<ul>
    {% for page in pages %}
        <li><a href="{{ page.get_absolute_url }}">{{ page.title }}</a></li>
    {% empty %}
        <li>No results</li>
    {% endfor %}
</ul>
{% endblock %}
