{% extends "powerwiki/base.html" %}

{% block wiki_content %}
  {% if wikis %}
    <table>
    {% for wiki in wikis %}
      <tr>
        <td><a href="{{ wiki.get_absolute_url }}">{{ wiki.title }}</a></td>
        <td>{{ wiki.description|default:"&nbsp;" }}</td>
      </tr>
    {% endfor %}
    </table>
  {% else %}
    <p>There are no wikis available.</p>
  {% endif %}
{% endblock %}
