{% extends "powerwiki/show.html" %}

{% block wiki_content %}
<div class="powerwiki__asset">
    <img src="{{ asset.image.url }}">

    {% if edit_url %}
    <ul>
    <li><a href="{{ edit_url }}">{{ edit_label }}</a></li>
    </ul>
    {% endif %}
</div>
{% endblock %}
